<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 02.04.2015
 */
namespace skeeks\modules\cms\shop\components;
use skeeks\cms\helpers\UrlHelper;
use skeeks\modules\cms\shop\models\ShopBasket;
use skeeks\modules\cms\shop\models\ShopFuser;
use yii\helpers\ArrayHelper;

/**
 * Class Shop
 * @package skeeks\modules\cms\shop\components
 */
class Cart extends \skeeks\cms\base\Component
{
    /**
     * @var ShopFuser
     */
    public $shopFuser = null;

    public $sessionFuserName = 'sx_fuser';

    public function init()
    {
        parent::init();

        if (\Yii::$app->user->isGuest)
        {
            //Проверка сессии
            if (\Yii::$app->getSession()->offsetExists($this->sessionFuserName))
            {
                $fuserId = \Yii::$app->getSession()->get($this->sessionFuserName);
                $shopFuser = ShopFuser::find()->where(['id' => $fuserId])->one();
                //Поиск юзера
                if ($shopFuser)
                {
                    $this->shopFuser = $shopFuser;
                }
            }
        } else
        {
            $this->shopFuser = ShopFuser::fetchByUser(\Yii::$app->cms->getAuthUser());

            //Если у авторизовнного пользоывателя уже есть пользователь корзины
            if ($this->shopFuser)
            {
                //Проверка сессии, а было ли чего то в корзине
                if (\Yii::$app->getSession()->offsetExists($this->sessionFuserName))
                {
                    $fuserId = \Yii::$app->getSession()->get($this->sessionFuserName);
                    $shopFuser = ShopFuser::find()->where(['id' => $fuserId])->one();

                    //Поиск юзера
                    /**
                     * @var $shopFuser ShopFuser
                     */
                    if ($shopFuser)
                    {
                        $this->shopFuser->addBaskets($shopFuser->getShopBaskets()->all());
                    }

                    \Yii::$app->getSession()->remove($this->sessionFuserName);
                }
            } else
            {
                //Проверка сессии, а было ли чего то в корзине
                if (\Yii::$app->getSession()->offsetExists($this->sessionFuserName))
                {
                    $fuserId = \Yii::$app->getSession()->get($this->sessionFuserName);
                    $shopFuser = ShopFuser::find()->where(['id' => $fuserId])->one();
                    //Поиск юзера
                    /**
                     * @var $shopFuser ShopFuser
                     */
                    if ($shopFuser)
                    {
                        $shopFuser->user_id = \Yii::$app->cms->getAuthUser()->id;
                        $shopFuser->save();
                    }

                    \Yii::$app->getSession()->remove($this->sessionFuserName);
                }
            }

        }
    }


    /**
     * Загрузить и создать пользователя корзины. Если нет.
     *
     * @return $this
     */
    public function loadShopFuser()
    {
        if ($this->shopFuser)
        {
            return $this;
        }

        if (\Yii::$app->user->isGuest)
        {
            $shopFuser = new ShopFuser([
                'user_id' => \Yii::$app->cms->getAuthUser()->id
            ]);

            $shopFuser->save();

            \Yii::$app->getSession()->set($this->sessionFuserName, $shopFuser->id);
            $this->shopFuser = $shopFuser;
        } else
        {
            $shopFuser = new ShopFuser([
                'user_id' => \Yii::$app->cms->getAuthUser()->id
            ]);

            $shopFuser->save();

            $this->shopFuser = $shopFuser;
        }

        return $this;
    }

    /**
     * @return array|\yii\db\ActiveQuery
     */
    public function findShopBaskets()
    {
        if (!$this->shopFuser)
        {
            return null;
        }

        return $this->shopFuser->getShopBaskets()->where(['order_id' => null]);
    }

    /**
     * @return ShopBasket[]
     */
    public function getShopBaskets()
    {
        if (!$this->shopFuser)
        {
            return [];
        }

        return $this->findShopBaskets()->all();
    }

    /**
     * Количество позиций в коризне
     *
     * @return int
     */
    public function countBaskets()
    {
        if ($this->getShopBaskets())
        {
            return count($this->getShopBaskets());
        }

        return 0;
    }


    /**
     * @return int
     */
    public function countProducts()
    {
        $result = 0;

        if ($baskets = $this->getShopBaskets())
        {
            foreach ($baskets as $basket)
            {
                $result = $result + $basket->quantity;
            }
        }

        return $result;
    }
    /**
     * Стоимость корзины.
     *
     * @return \skeeks\modules\cms\money\Money
     */
    public function cost()
    {
        $money = \Yii::$app->money->newMoney();

        if (!$this->countBaskets())
        {
            return $money;
        }

        if ($baskets = $this->getShopBaskets())
        {
            foreach ($baskets as $basket)
            {
                $money = $money->add($basket->price());
            }
        }

        return $money;
    }
}