<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 15.03.2015
 */
namespace skeeks\modules\cms\shop\controllers;
use skeeks\cms\base\Controller;
use skeeks\cms\helpers\RequestResponse;
use skeeks\cms\helpers\UrlHelper;
use skeeks\cms\models\forms\SignupForm;
use skeeks\modules\cms\form\models\Form;
use skeeks\modules\cms\form\models\FormField;
use skeeks\modules\cms\form\models\FormSendMessage;
use skeeks\modules\cms\shop\models\ShopOrder;
use skeeks\modules\cms\shop\models\ShopPaySystem;
use skeeks\modules\cms\shop\models\ShopPersonType;
use yii\base\Exception;
use yii\base\UserException;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use yii\web\Response;
use yii\widgets\ActiveForm;

/**
 * Class BackendController
 * @package skeeks\modules\cms\form\controllers
 */
class BackendController extends Controller
{
    /**
     * @return array
     */
    public function behaviors()
    {
        return ArrayHelper::merge(parent::behaviors(), [

            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'validate' => ['post'],
                    'submit' => ['post'],
                ],
            ],
        ]);
    }


    /**
     * Процесс отправки формы
     * @return array
     */
    public function actionCreateOrder()
    {
        if (\Yii::$app->request->isAjax && !\Yii::$app->request->isPjax)
        {
            \Yii::$app->response->format = Response::FORMAT_JSON;

            $rr = new RequestResponse();

            try
            {
                if (!$formId = \Yii::$app->request->post(Form::FROM_PARAM_ID_NAME))
                {
                    throw new UserException('Не найден параметр формы');
                }

                /**
                 * @var $modelForm Form
                 */
                $modelForm  = Form::find()->where(['id' => $formId])->one();
                $model      = $modelForm->createValidateModel();

                if ($model->load(\Yii::$app->request->post()) && $model->validate())
                {
                    $shopPersonTypeId = (int) \Yii::$app->request->post('shopPersonType');
                    if (!$shopPersonTypeId)
                    {
                        throw new UserException('Не передан параметр тип плательщика');
                    }

                    $shopPersonType = ShopPersonType::findOne($shopPersonTypeId);
                    if (!$shopPersonType)
                    {
                        throw new UserException('Не указан плательщик');
                    }

                    $shopPaySystemId = (int) \Yii::$app->request->post('shopPaySystemId');
                    $shopPaySystem = ShopPaySystem::findOne($shopPaySystemId);
                    if (!$shopPaySystem)
                    {
                        throw new UserException('Не указана система оплаты');
                    }

                    if (!$model->email)
                    {
                        throw new UserException('Не указан email.');
                    }

                    if (!\Yii::$app->shop->cart->countProducts())
                    {
                        throw new UserException('Нет товаров в корзине.');
                    }


                    //Пользователь авторизован или нет.
                    if (\Yii::$app->user->isGuest)
                    {
                        $registerForm = new SignupForm();
                        $registerForm->setScenario(SignupForm::SCENARION_ONLYEMAIL);
                        $registerForm->email = $model->email;

                        $user = $registerForm->signup();

                        //Проверим может что то можно заполнить в профиле пользователя.
                        if ($user)
                        {
                            foreach ($model->attributeValues() as $code => $value)
                            {
                                if ($user->hasAttribute($code))
                                {
                                    $user->{$code} = $value;
                                }
                            }

                            $user->save(false);
                        } else
                        {
                            throw new UserException("Не удалось зарегистрировать пользователя, возможно пользователь с указанным email адресом уже зарегистрирован.");
                        }

                        //Авторизация пользователя
                        \Yii::$app->user->login($user, 0);

                    } else
                    {
                        $user = \Yii::$app->cms->getAuthUser();
                    }

                    $cost       = \Yii::$app->shop->cart->cost();
                    $currency   = $cost->getCurrency();

                    //Создание нового заказа
                    $shopOrder = new ShopOrder([
                        'person_type_id'    => $shopPersonType->id,
                        'price'             => $cost->getAmount() / $currency->getSubUnit(),
                        'currency'          => (string) $currency,
                        'user_id'           => $user->id,
                        'pay_system_id'     => $shopPaySystem->id,
                        'data'              => $model->attributeLabelsValues(),
                    ]);

                    if (!$shopOrder->save())
                    {
                        $result = [];
                        $errors = $shopOrder->getErrors();
                        foreach ($errors as $errorData)
                        {
                            $result[] = implode(', ', $errorData);
                        }

                        throw new UserException('Не получилось оформить заказ: ' . implode('; ', $result));
                    }

                    //Переписать позиции корзины
                    if ($shopBaskets = \Yii::$app->shop->cart->getShopBaskets())
                    {
                        foreach ($shopBaskets as $shopBasket)
                        {
                            $shopBasket->order_id = $shopOrder->id;
                            $shopBasket->save();
                        }
                    }

                    //Отправка сообщения с формы
                    //Все проверки прошли, формируем модель отправленного сообщения и сохраняем ее
                    $modelFormSendMessage = new FormSendMessage();

                    $modelFormSendMessage->data_values = $model->attributeValues();
                    $modelFormSendMessage->data_labels = $model->attributeLabels();

                    $modelFormSendMessage->page_url = \Yii::$app->request->referrer;
                    $modelFormSendMessage->form_id = $formId;

                    if ($modelFormSendMessage->save())
                    {
                        $modelFormSendMessage->notify();

                        $rr->success = true;
                        $rr->message = 'Успешно отправлена';

                    } else
                    {
                        $rr->message = 'Не удалось сохранить сообщение в базу';
                    }


                    $rr->success = true;
                    $rr->message = 'Заказ успешно создан';
                    \Yii::$app->session->setFlash("success", 'Заказ успешно создан');

                    return $this->redirect( UrlHelper::construct('shop/order/view', [
                        'id' => $shopOrder->id,
                        'sx-new' => '1'
                    ])->toString() );

                } else
                {
                    $rr->message = 'Форма заполнена неправильно';
                }

            } catch(\Exception $e)
            {
                $rr->message = $e->getMessage();
            }

            return (array) $rr;

        }
    }
}