<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 01.04.2015
 */
use yii\db\Schema;
use yii\db\Migration;

class m150401_110600_create_table__shop_order extends Migration
{
    public function safeUp()
    {
        $tableExist = $this->db->getTableSchema("{{%shop_order}}", true);
        if ($tableExist)
        {
            return true;
        }

        $tableOptions = null;
        if ($this->db->driverName === 'mysql') {
            $tableOptions = 'CHARACTER SET utf8 COLLATE utf8_general_ci ENGINE=InnoDB';
        }

        $this->createTable("{{%shop_order}}", [
            'id'                    => Schema::TYPE_PK,

            'created_by'            => Schema::TYPE_INTEGER . ' NULL',
            'updated_by'            => Schema::TYPE_INTEGER . ' NULL',

            'created_at'            => Schema::TYPE_INTEGER . ' NULL',
            'updated_at'            => Schema::TYPE_INTEGER . ' NULL',

            'person_type_id'        => Schema::TYPE_INTEGER . ' NULL',

            'payed'                 => Schema::TYPE_SMALLINT . '(1) NOT NULL DEFAULT 0',
            'payed_at'              => Schema::TYPE_INTEGER . ' NULL',

            'canceled'              => Schema::TYPE_SMALLINT . '(1) NOT NULL DEFAULT 0',
            'canceled_at'           => Schema::TYPE_INTEGER . ' NULL',
            'reason_canceled'       => Schema::TYPE_STRING . '(255) NULL',

            'status_code'           => 'char(1) NOT NULL DEFAULT "N"',
            'status_at'             => Schema::TYPE_INTEGER . ' NULL',

            'price'                 => Schema::TYPE_DECIMAL . '(18,2)',
            'currency'              => Schema::TYPE_STRING. '(3) DEFAULT "RUB"',

            'price_delivery'        => Schema::TYPE_DECIMAL . '(18,2) NULL',
            'allow_delivery'        => Schema::TYPE_SMALLINT . '(1) NOT NULL DEFAULT 0',
            'allow_delivery_at'     => Schema::TYPE_INTEGER . ' NULL',

            'discount_value'        => Schema::TYPE_DECIMAL . '(18,2) NULL',
            'user_id'               => Schema::TYPE_INTEGER . ' NOT NULL',

            'pay_system_id'         => Schema::TYPE_INTEGER . ' NULL',
            'delivery_id'           => Schema::TYPE_INTEGER . ' NULL',

            'user_description'      => Schema::TYPE_STRING . '(255) NULL',
            'additional_info'       => Schema::TYPE_STRING . '(255) NULL',

            'ps_status'             => Schema::TYPE_STRING . '(1) NULL',
            'ps_status_code'        => Schema::TYPE_STRING . '(5) NULL',
            'ps_status_description' => Schema::TYPE_STRING . '(255) NULL',
            'ps_status_message'     => Schema::TYPE_STRING . '(255) NULL',
            'ps_sum'                => Schema::TYPE_DECIMAL . '(18,2) NULL',
            'ps_currency'           => Schema::TYPE_STRING. '(3) NULL',
            'ps_respose_at'         => Schema::TYPE_INTEGER. ' NULL',
            'data'                  => Schema::TYPE_TEXT. ' NULL',

            'comments'              => Schema::TYPE_TEXT. ' NULL',
            'tax_value'             => Schema::TYPE_DECIMAL . '(18,2) NULL',

            'locked_by'            => Schema::TYPE_INTEGER . ' NULL',
            'locked_at'            => Schema::TYPE_INTEGER . ' NULL',

            'reserved'            => Schema::TYPE_SMALLINT . '(1) NOT NULL DEFAULT 0',

        ], $tableOptions);

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(updated_by);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(created_by);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(created_at);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(updated_at);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(person_type_id);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(payed);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(payed_at);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(status_code);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(status_at);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(canceled);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(canceled_at);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(reason_canceled);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(price);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(currency);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(price_delivery);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(allow_delivery);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(allow_delivery_at);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(discount_value);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(user_id);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(pay_system_id);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(delivery_id);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(user_description);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(additional_info);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_status);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_status_code);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_status_description);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_status_message);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_sum);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_currency);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(ps_respose_at);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(tax_value);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(locked_by);");
        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(locked_at);");

        $this->execute("ALTER TABLE {{%shop_order}} ADD INDEX(reserved);");

        $this->execute("ALTER TABLE {{%shop_order}} COMMENT = 'Заказы';");

        $this->addForeignKey(
            'shop_order_created_by', "{{%shop_order}}",
            'created_by', '{{%cms_user}}', 'id', 'SET NULL', 'SET NULL'
        );

        $this->addForeignKey(
            'shop_order_updated_by', "{{%shop_order}}",
            'updated_by', '{{%cms_user}}', 'id', 'SET NULL', 'SET NULL'
        );

        $this->addForeignKey(
            'shop_order_person_type_id', "{{%shop_order}}",
            'person_type_id', '{{%shop_person_type}}', 'id', 'RESTRICT', 'RESTRICT'
        );

        $this->addForeignKey(
            'shop_order__user_id', "{{%shop_order}}",
            'user_id', '{{%cms_user}}', 'id', 'RESTRICT', 'RESTRICT'
        );

        $this->addForeignKey(
            'shop_order__status_code', "{{%shop_order}}",
            'status_code', '{{%shop_order_status}}', 'code', 'RESTRICT', 'RESTRICT'
        );


        $this->addForeignKey(
            'shop_order__locked_by', "{{%shop_order}}",
            'locked_by', '{{%cms_user}}', 'id', 'RESTRICT', 'RESTRICT'
        );
    }

    public function safeDown()
    {
        $this->dropForeignKey("shop_order_updated_by", "{{%shop_order}}");
        $this->dropForeignKey("shop_order_updated_by", "{{%shop_order}}");
        $this->dropForeignKey("shop_order_person_type_id", "{{%shop_order}}");
        $this->dropForeignKey("shop_order__user_id", "{{%shop_order}}");
        $this->dropForeignKey("shop_order__status_code", "{{%shop_order}}");
        $this->dropForeignKey("shop_order__locked_by", "{{%shop_order}}");

        $this->dropTable("{{%shop_order}}");
    }
}
