<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 01.04.2015
 */
use yii\db\Schema;
use yii\db\Migration;

class m150401_112818_create_table__shop_basket extends Migration
{
    public function up()
    {
        $tableExist = $this->db->getTableSchema("{{%shop_basket}}", true);
        if ($tableExist)
        {
            return true;
        }

        $tableOptions = null;
        if ($this->db->driverName === 'mysql') {
            $tableOptions = 'CHARACTER SET utf8 COLLATE utf8_general_ci ENGINE=InnoDB';
        }

        $this->createTable("{{%shop_basket}}", [
            'id'                    => Schema::TYPE_PK,

            'created_by'            => Schema::TYPE_INTEGER . ' NULL',
            'updated_by'            => Schema::TYPE_INTEGER . ' NULL',

            'created_at'            => Schema::TYPE_INTEGER . ' NULL',
            'updated_at'            => Schema::TYPE_INTEGER . ' NULL',

            'fuser_id'              => Schema::TYPE_INTEGER . ' NOT NULL',
            'order_id'              => Schema::TYPE_INTEGER . ' NULL',
            'product_id'            => Schema::TYPE_INTEGER . ' NULL',

            'price'                 => Schema::TYPE_DECIMAL . '(18,2)',
            'currency'              => Schema::TYPE_STRING. '(3)',

            'weight'                => 'double (18,2)',
            'quantity'              => 'double (18,2) DEFAULT "0.00"',

            'name'                  => Schema::TYPE_STRING . '(255) NOT NULL',

            'can_buy'               => Schema::TYPE_SMALLINT . ' NOT NULL DEFAULT 1',
            'notes'                 => Schema::TYPE_STRING . '(255) NULL',
            'detail_page_url'       => Schema::TYPE_TEXT . ' NULL',

            'discount_price'        => Schema::TYPE_DECIMAL . '(18,2) NOT NULL DEFAULT "0.00"',
            'discount_name'         => Schema::TYPE_STRING . '(255) NULL',
            'discount_value'        => Schema::TYPE_STRING . '(32) NULL',
            'discount_coupon'       => Schema::TYPE_STRING . '(32) NULL',

            'vat_rate'              => Schema::TYPE_DECIMAL . '(18,2) NOT NULL DEFAULT "0.00"',

            'reserved'              => Schema::TYPE_SMALLINT . ' NOT NULL DEFAULT 0',
            'reserved_quantity'     => 'double NULL',
            'deducted'              => Schema::TYPE_SMALLINT . ' NOT NULL DEFAULT 0',
            'custom_price'          => Schema::TYPE_SMALLINT . ' NOT NULL DEFAULT 0',

        ], $tableOptions);

        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(updated_by);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(created_by);");

        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(created_at);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(updated_at);");

        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(fuser_id);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(order_id);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(product_id);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(price);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(currency);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(weight);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(quantity);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(name);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(can_buy);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(notes);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(discount_price);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(discount_name);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(discount_value);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(discount_coupon);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(vat_rate);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(reserved);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(reserved_quantity);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(deducted);");
        $this->execute("ALTER TABLE {{%shop_basket}} ADD INDEX(custom_price);");

        $this->execute("ALTER TABLE {{%shop_basket}} ADD UNIQUE(fuser_id,product_id,order_id);");

        $this->execute("ALTER TABLE {{%shop_basket}} COMMENT = 'Позиция в корзине';");

        $this->addForeignKey(
            'shop_basket_created_by', "{{%shop_basket}}",
            'created_by', '{{%cms_user}}', 'id', 'SET NULL', 'SET NULL'
        );

        $this->addForeignKey(
            'shop_basket_updated_by', "{{%shop_basket}}",
            'updated_by', '{{%cms_user}}', 'id', 'SET NULL', 'SET NULL'
        );

        $this->addForeignKey(
            'shop_basket_fuser_id', "{{%shop_basket}}",
            'fuser_id', '{{%shop_fuser}}', 'id', 'CASCADE', 'CASCADE'
        );

        $this->addForeignKey(
            'shop_basket_order_id', "{{%shop_basket}}",
            'order_id', '{{%shop_order}}', 'id', 'SET NULL', 'SET NULL'
        );

        $this->addForeignKey(
            'shop_basket_product_id', "{{%shop_basket}}",
            'product_id', '{{%catalog_product}}', 'id', 'SET NULL', 'SET NULL'
        );
    }

    public function down()
    {
        $this->dropForeignKey("shop_basket_created_by", "{{%shop_basket}}");
        $this->dropForeignKey("shop_basket_updated_by", "{{%shop_basket}}");

        $this->dropForeignKey("shop_basket_fuser_id", "{{%shop_basket}}");
        $this->dropForeignKey("shop_basket_order_id", "{{%shop_basket}}");
        $this->dropForeignKey("shop_basket_product_id", "{{%shop_basket}}");

        $this->dropTable("{{%shop_basket}}");
    }
}
