<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 02.04.2015
 */
namespace skeeks\modules\cms\shop\models;
use skeeks\cms\models\behaviors\HasStatus;
use skeeks\cms\models\behaviors\HasStatusBoolean;
use skeeks\cms\models\Core;
use skeeks\modules\cms\catalog\models\Product;
use skeeks\modules\cms\money\Currency;
use skeeks\modules\cms\money\Money;
use \Yii;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "{{%shop_basket}}".
 *
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property integer $fuser_id
 * @property integer $order_id
 * @property integer $product_id
 * @property string $price
 * @property string $currency
 * @property double $weight
 * @property double $quantity
 * @property string $name
 * @property integer $can_buy
 * @property string $notes
 * @property string $detail_page_url
 * @property string $discount_price
 * @property string $discount_name
 * @property string $discount_value
 * @property string $discount_coupon
 * @property string $vat_rate
 * @property integer $reserved
 * @property double $reserved_quantity
 * @property integer $deducted
 * @property integer $custom_price
 *
 * @property CatalogProduct $product
 * @property CmsUser $createdBy
 * @property ShopFuser $fuser
 * @property ShopOrder $order
 * @property CmsUser $updatedBy
 */
class ShopBasket extends Core
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%shop_basket}}';
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return ArrayHelper::merge(parent::behaviors(), [
            HasStatusBoolean::className() =>
            [
                'class' => HasStatusBoolean::className(),
            ]
        ]);
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return array_merge(parent::attributeLabels(), [
            'id' => Yii::t('app', 'ID'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'created_at' => Yii::t('app', 'Created At'),
            'updated_at' => Yii::t('app', 'Updated At'),
            'fuser_id' => Yii::t('app', 'Fuser ID'),
            'order_id' => Yii::t('app', 'Order ID'),
            'product_id' => Yii::t('app', 'Product ID'),
            'price' => Yii::t('app', 'Price'),
            'currency' => Yii::t('app', 'Currency'),
            'weight' => Yii::t('app', 'Weight'),
            'quantity' => Yii::t('app', 'Quantity'),
            'name' => Yii::t('app', 'Name'),
            'can_buy' => Yii::t('app', 'Can Buy'),
            'notes' => Yii::t('app', 'Notes'),
            'detail_page_url' => Yii::t('app', 'Detail Page Url'),
            'discount_price' => Yii::t('app', 'Discount Price'),
            'discount_name' => Yii::t('app', 'Discount Name'),
            'discount_value' => Yii::t('app', 'Discount Value'),
            'discount_coupon' => Yii::t('app', 'Discount Coupon'),
            'vat_rate' => Yii::t('app', 'Vat Rate'),
            'reserved' => Yii::t('app', 'Reserved'),
            'reserved_quantity' => Yii::t('app', 'Reserved Quantity'),
            'deducted' => Yii::t('app', 'Deducted'),
            'custom_price' => Yii::t('app', 'Custom Price'),
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios[self::SCENARIO_DEFAULT];

        $scenarios['create'] = $scenarios[self::SCENARIO_DEFAULT];
        $scenarios['update'] = $scenarios[self::SCENARIO_DEFAULT];

        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['created_by', 'updated_by', 'created_at', 'updated_at', 'fuser_id', 'order_id', 'product_id', 'can_buy', 'reserved', 'deducted', 'custom_price'], 'integer'],
            [['fuser_id', 'name'], 'required'],
            [['price', 'weight', 'quantity', 'discount_price', 'vat_rate', 'reserved_quantity'], 'number'],
            [['detail_page_url'], 'string'],
            [['currency'], 'string', 'max' => 3],
            [['name', 'notes', 'discount_name'], 'string', 'max' => 255],
            [['discount_value', 'discount_coupon'], 'string', 'max' => 32]
        ]);
    }


    /**
     * @return Money
     */
    public function price()
    {
        if ($this->price)
        {
            return Money::fromString((string) $this->price, $this->currency());
        }

        return Money::fromString((string) 0, $this->currency());
    }

    /**
     * @return Currency
     */
    public function currency()
    {
        if ($currency = $this->getAttribute($this->currency))
        {
            return Currency::getInstance($currency);
        }

        return Currency::getInstance(\Yii::$app->money->currency);
    }


    protected $_product = null;

    /**
     * @return Product
     */
    public function product()
    {
        if ($this->_product === null)
        {
            $this->_product = $this->getProduct()->one();
        }

        return $this->_product;
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProduct()
    {
        return $this->hasOne(Product::className(), ['id' => 'product_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(CmsUser::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFuser()
    {
        return $this->hasOne(ShopFuser::className(), ['id' => 'fuser_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrder()
    {
        return $this->hasOne(ShopOrder::className(), ['id' => 'order_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(CmsUser::className(), ['id' => 'updated_by']);
    }


    /**
     * Обновить позицию.
     *
     * @return $this
     */
    public function recalculate()
    {
        /**
         * @var $product Product
         */
        $product = $this->getProduct()->one();
        if (!$product)
        {
            return $this;
        }

        $cost   = $product->cost(); //цена продукта
        $money  = $cost->multiply($this->quantity); //умножаем на количество

        $this->price        = $money->getAmount() / $money->getCurrency()->getSubUnit();
        $this->currency     = (string) $money->getCurrency();

        $this->save();

        return $this;
    }
}