<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 02.04.2015
 */
namespace skeeks\modules\cms\shop\models;
use skeeks\cms\models\behaviors\Serialize;
use skeeks\cms\models\User;
use skeeks\cms\helpers\UrlHelper;
use skeeks\cms\models\behaviors\HasStatus;
use skeeks\cms\models\behaviors\HasStatusBoolean;
use skeeks\cms\models\Core;
use skeeks\modules\cms\money\Currency;
use skeeks\modules\cms\money\Money;
use \Yii;
use yii\db\BaseActiveRecord;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "{{%shop_order}}".
 *
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property integer $person_type_id
 * @property integer $payed
 * @property integer $payed_at
 * @property integer $canceled
 * @property integer $canceled_at
 * @property string $reason_canceled
 * @property string $status_code
 * @property integer $status_at
 * @property string $price
 * @property string $currency
 * @property string $price_delivery
 * @property integer $allow_delivery
 * @property integer $allow_delivery_at
 * @property string $discount_value
 * @property integer $user_id
 * @property integer $pay_system_id
 * @property integer $delivery_id
 * @property string $user_description
 * @property string $additional_info
 * @property string $ps_status
 * @property string $ps_status_code
 * @property string $ps_status_description
 * @property string $ps_status_message
 * @property string $ps_sum
 * @property string $ps_currency
 * @property integer $ps_respose_at
 * @property string $data
 * @property string $comments
 * @property string $tax_value
 * @property integer $locked_by
 * @property integer $locked_at
 * @property integer $reserved
 *
 * @property ShopBasket[] $shopBaskets
 * @property User $lockedBy
 * @property User $createdBy
 * @property ShopPersonType $personType
 * @property User $updatedBy
 * @property ShopOrderStatus $statusCode
 * @property User $user
 */
class ShopOrder extends Core
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%shop_order}}';
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return ArrayHelper::merge(parent::behaviors(), [
            HasStatusBoolean::className() =>
            [
                'class' => HasStatusBoolean::className(),
            ],
            Serialize::className() =>
            [
                'class' => Serialize::className(),
                'fields' => ['data']
            ]
        ]);
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return array_merge(parent::attributeLabels(), [
            'id' => Yii::t('app', 'ID'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'created_at' => Yii::t('app', 'Created At'),
            'updated_at' => Yii::t('app', 'Updated At'),
            'person_type_id' => Yii::t('app', 'Person Type ID'),
            'payed' => Yii::t('app', 'Оплачен'),
            'payed_at' => Yii::t('app', 'Payed At'),
            'canceled' => Yii::t('app', 'Отменен'),
            'canceled_at' => Yii::t('app', 'Canceled At'),
            'reason_canceled' => Yii::t('app', 'Reason Canceled'),
            'status' => Yii::t('app', 'Status'),
            'price' => Yii::t('app', 'Price'),
            'currency' => Yii::t('app', 'Currency'),
            'price_delivery' => Yii::t('app', 'Price Delivery'),
            'allow_delivery' => Yii::t('app', 'Доставка разрешена?'),
            'discount_value' => Yii::t('app', 'Discount Value'),
            'user_id' => Yii::t('app', 'User ID'),
            'pay_system_id' => Yii::t('app', 'Pay System ID'),
            'delivery_id' => Yii::t('app', 'Delivery ID'),
            'user_description' => Yii::t('app', 'User Description'),
            'additional_info' => Yii::t('app', 'Additional Info'),
            'ps_status' => Yii::t('app', 'Ps Status'),
            'ps_status_code' => Yii::t('app', 'Ps Status Code'),
            'ps_status_description' => Yii::t('app', 'Ps Status Description'),
            'ps_status_message' => Yii::t('app', 'Ps Status Message'),
            'ps_sum' => Yii::t('app', 'Ps Sum'),
            'ps_currency' => Yii::t('app', 'Ps Currency'),
            'ps_respose_at' => Yii::t('app', 'Ps Respose At'),
            'comments' => Yii::t('app', 'Comments'),
            'tax_value' => Yii::t('app', 'Tax Value'),
            'status_code' => Yii::t('app', 'Статус'),
            'data' => Yii::t('app', 'Данные заказа'),
            'reason_canceled' => Yii::t('app', 'Причина отмены'),
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios[self::SCENARIO_DEFAULT];

        $scenarios['create'] = $scenarios[self::SCENARIO_DEFAULT];
        $scenarios['update'] = $scenarios[self::SCENARIO_DEFAULT];

        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function init()
    {
        parent::init();

        $this->on(BaseActiveRecord::EVENT_BEFORE_INSERT,    [$this, "checkSave"]);
        $this->on(BaseActiveRecord::EVENT_BEFORE_UPDATE,    [$this, "checkSave"]);
    }

    public function checkSave()
    {
        if ($this->getOldAttribute('status_code') != $this->getAttribute('status_code'))
        {
            $this->status_at = \Yii::$app->formatter->asTimestamp(time());
        }

        if ($this->getOldAttribute('payed') != $this->getAttribute('payed'))
        {
            $this->payed_at = \Yii::$app->formatter->asTimestamp(time());
        }

        if ($this->getOldAttribute('canceled') != $this->getAttribute('canceled'))
        {
            $this->canceled_at = \Yii::$app->formatter->asTimestamp(time());
        }
    }


    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['created_by', 'updated_by', 'created_at', 'updated_at', 'person_type_id', 'payed', 'payed_at', 'canceled', 'canceled_at', 'status_at', 'allow_delivery', 'allow_delivery_at', 'user_id', 'pay_system_id', 'delivery_id', 'ps_respose_at', 'locked_by', 'locked_at', 'reserved'], 'integer'],
            [['price', 'price_delivery', 'discount_value', 'ps_sum', 'tax_value'], 'number'],
            [['user_id'], 'required'],
            [['data'], 'safe'],
            [['canceled'], 'validateCanceled'],
            [['comments'], 'string'],
            [['reason_canceled', 'user_description', 'additional_info', 'ps_status_description', 'ps_status_message'], 'string', 'max' => 255],
            [['status_code', 'ps_status'], 'string', 'max' => 1],
            [['currency', 'ps_currency'], 'string', 'max' => 3],
            [['ps_status_code'], 'string', 'max' => 5],

            ['status_at', 'default', 'value' => function(ShopOrder $model, $attribute)
            {
                if (!$model->$attribute)
                {
                    return \Yii::$app->formatter->asTimestamp(time());
                }
            }],

        ]);
    }

    /**
     * Правильная валидация видео ссылки.
     *
     * @param $attribute
     */
    public function validateCanceled($attribute)
    {
        if ($this->$attribute == 1)
        {
            if (!$this->reason_canceled)
            {
                $this->addError($attribute, "Укажите причину отмены");
            }
        }
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopBaskets()
    {
        return $this->hasMany(ShopBasket::className(), ['order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLockedBy()
    {
        return $this->hasOne(CmsUser::className(), ['id' => 'locked_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPersonType()
    {
        return $this->hasOne(ShopPersonType::className(), ['id' => 'person_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPaySystem()
    {
        return $this->hasOne(ShopPaySystem::className(), ['id' => 'pay_system_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getStatusCode()
    {
        return $this->hasOne(ShopOrderStatus::className(), ['code' => 'status_code']);
    }



    /**
     * @return Money
     */
    public function price()
    {
        if ($this->price)
        {
            return Money::fromString((string) $this->price, $this->currency());
        }

        return Money::fromString((string) 0, $this->currency());
    }

    /**
     * @return Currency
     */
    public function currency()
    {
        if ($currency = $this->getAttribute($this->currency))
        {
            return Currency::getInstance($currency);
        }

        return Currency::getInstance(\Yii::$app->money->currency);
    }

    /**
     * @return string
     */
    public function getPageUrl()
    {
        return UrlHelper::construct('shop/order/view', ['id' => $this->id])->toString();
    }
}