<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 21.04.2015
 */
namespace skeeks\modules\cms\shop\models;

use skeeks\cms\models\Core;
use skeeks\cms\models\User;
use skeeks\modules\cms\shop\models\ShopOrder;
use Yii;
use yii\base\UserException;
use yii\db\BaseActiveRecord;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "{{%shop_order_status}}".
 *
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property string $code
 * @property string $name
 * @property string $description
 * @property integer $priority
 * @property string $color
 *
 * @property ShopOrder[] $shopOrders
 * @property User $updatedBy
 * @property User $createdBy
 */
class ShopOrderStatus extends Core
{
    const STATUS_CODE_START  = "N";
    const STATUS_CODE_END    = "F";

    static public $protectedStatuses =
    [
        self::STATUS_CODE_START, self::STATUS_CODE_END
    ];

    /**
     * @inheritdoc
     */
    public function init()
    {
        parent::init();

        $this->on(BaseActiveRecord::EVENT_BEFORE_DELETE,    [$this, "checkDelete"]);
    }

    public function checkDelete()
    {
        if ($this->isProtected())
        {
            throw new UserException('Нельзя удалять этот статус');
        }
    }

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%shop_order_status}}';
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return ArrayHelper::merge(parent::behaviors(), []);
    }

    /**
     * Нельзя удалять и редактировать статус?
     * @return bool
     */
    public function isProtected()
    {
        if (in_array($this->code, (array) static::$protectedStatuses))
        {
            return true;
        }

        return false;
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return array_merge(parent::attributeLabels(), [
            'id' => Yii::t('app', 'ID'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'created_at' => Yii::t('app', 'Created At'),
            'updated_at' => Yii::t('app', 'Updated At'),
            'code' => Yii::t('app', 'Код'),
            'name' => Yii::t('app', 'Name'),
            'description' => Yii::t('app', 'Описание'),
            'priority' => Yii::t('app', 'Priority'),
            'color' => Yii::t('app', 'Color'),
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios[self::SCENARIO_DEFAULT];

        $scenarios['create'] = $scenarios[self::SCENARIO_DEFAULT];
        $scenarios['update'] = $scenarios[self::SCENARIO_DEFAULT];

        return $scenarios;
    }


    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['created_by', 'updated_by', 'created_at', 'updated_at', 'priority'], 'integer'],
            [['code', 'name'], 'required'],
            [['description'], 'string'],
            [['code'], 'string', 'max' => 1],
            [['name'], 'string', 'max' => 255],
            [['color'], 'string', 'max' => 32],
            [['code'], 'unique'],
            [['code'], 'validateCode']
        ]);
    }


    public function validateCode($attribute)
    {
        if(!preg_match('/^[A-Z]{1}$/', $this->$attribute))
        {
            $this->addError($attribute, 'Используйте только заглавные буквы латинского алфавита.');
        }
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrders()
    {
        return $this->hasMany(ShopOrder::className(), ['status_code' => 'code']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(CmsUser::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(CmsUser::className(), ['id' => 'created_by']);
    }
}