<?php
/**
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 02.04.2015
 */
namespace skeeks\modules\cms\shop\models;
use skeeks\cms\models\behaviors\HasStatus;
use skeeks\cms\models\behaviors\HasStatusBoolean;
use skeeks\cms\models\Core;
use skeeks\modules\cms\form\models\Form;
use \Yii;
use yii\helpers\ArrayHelper;
use yii\web\ErrorHandler;

/**
 * This is the model class for table "{{%shop_person_type}}".
 *
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property string $name
 * @property integer $active
 * @property integer $form_id
 *
 * @property ShopOrder[] $shopOrders
 * @property User $updatedBy
 * @property User $createdBy
 */
class ShopPersonType extends Core
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%shop_person_type}}';
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return ArrayHelper::merge(parent::behaviors(), [
            HasStatusBoolean::className() =>
            [
                'class' => HasStatusBoolean::className(),
            ]
        ]);
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return array_merge(parent::attributeLabels(), [
            'id' => Yii::t('app', 'ID'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'created_at' => Yii::t('app', 'Created At'),
            'updated_at' => Yii::t('app', 'Updated At'),
            'name' => Yii::t('app', 'Name'),
            'active' => Yii::t('app', 'Активность'),
            'form_id' => Yii::t('app', 'Форма с полями и данными'),
        ]);
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();

        $scenarios[self::SCENARIO_DEFAULT];

        $scenarios['create'] = $scenarios[self::SCENARIO_DEFAULT];
        $scenarios['update'] = $scenarios[self::SCENARIO_DEFAULT];

        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['created_by', 'updated_by', 'created_at', 'updated_at', 'active', 'form_id'], 'integer'],
            [['name'], 'required'],
            [['name'], 'string', 'max' => 255]
        ]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrders()
    {
        return $this->hasMany(ShopOrder::className(), ['person_type_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getForm()
    {
        return $this->hasOne(Form::className(), ['id' => 'form_id']);
    }

    /**
     * @return Form
     */
    public function form()
    {
        return $this->getForm()->one();
    }

    /**
     * @return string
     */
    public function renderForm()
    {
        /**
         * @var $moduleForm \skeeks\modules\cms\shop\Module
         */
        //$moduleForm = \Yii::$app->getModule('shop');
        $moduleForm = \Yii::$app->getModule('form');
        $formModel = $this->form();

        try
        {
            $model = $formModel->createValidateModel();
            if (!\Yii::$app->user->isGuest)
            {
                $model->setAttributes(\Yii::$app->cms->getAuthUser()->toArray(), true);
            }

            return \Yii::$app->view->render('@skeeks/modules/cms/shop/views/cart-form', [
                'shopPersonType'    => $this,
                'shopPaySystems'    => $this->fetchPaySystems(),
                'module'            => $moduleForm,
                'modelForm'         => $formModel,
                'fields'            => $formModel->fields(),
                'model'             => $model
            ]);

        } catch (\Exception $e)
        {
            ob_end_clean();
            ErrorHandler::convertExceptionToError($e);
            return 'Ошибка рендеринга формы: ' . $e->getMessage();
        }
    }

    /**
     * TODO: добавить выборку платежных систем которые подходят для текущего плательщика.
     * @return ShopPaySystem[]
     */
    public function fetchPaySystems()
    {
        return ShopPaySystem::find()->where(['active' => 1])->all();
    }
}